#include "network-application.h"

void NetworkApplication::InitSocketWithRobotApplication(SInt32 _id, int port_base, int _max_content_size) 
  //    id(_id), max_content_size(_max_content_size)
  {
    id = _id;
    max_content_size = _max_content_size;

    OpenBlockingUDPSocketServerSide(socket, port_base + (id+1));
     
    msg_content = new char[max_content_size];
    received_data = new char[max_content_size];
  }

void NetworkApplication::GetStartInfoFromRobotAppl()
  { 
    fprintf(stderr,"Waiting for start info\n");
    ServerReceiveDataFromSocketBlocking(socket, (char *)&msg_start, msg_size, sizeof(RobotStartPacket));

    if( msg_size != sizeof(RobotStartPacket) )
      {
	fprintf(stderr, "WRONG SIZE FROM ROBOT START PACKET (size %d instead of %d)\n", msg_size, sizeof(RobotStartPacket));
	exit(-1);
      }

    if( msg_start.type != START_FROM_ROBOT_PROCESS )
      {
	fprintf(stderr, "WRONG START PACKET (type %d)\n", msg_start.type);
	exit(-1);
      }
    else
      fprintf(stderr, "[%d] Received START from robot %d (%s)\n", id, msg_start.id, msg_start.robot_type);
  }

vector<string> NetworkApplication::GetNewInputDataFromRobotApp(Real current_time_slice, int num_robots)
{
   char num_msgs;
   vector<string> vecRecv(num_robots,"");
   ServerReceiveDataFromSocketBlocking(socket, &num_msgs, msg_size, 1);
fprintf(stderr,"Node %d receiving %d messages from robot simulator\n",id,num_msgs);
   for (char i = 0; i < num_msgs; i++)
   {
      ReceiveData(current_time_slice);
      if (msg_header.to == -1)
	msg_header.to = this->id;
      string str_content(msg_content + sizeof(ApplicationPacket),msg_size-sizeof(ApplicationPacket));
      vecRecv[msg_header.to] = str_content;
   }
   return vecRecv;
}

void NetworkApplication::ReceiveData(Real current_time_slice)
{
   ServerReceiveDataFromSocketBlocking(socket, &msg_content, msg_size, MAX_MSG_SIZE + sizeof(ApplicationPacket));
   memcpy(&msg_header,msg_content,sizeof(ApplicationPacket));
   fprintf(stderr, "[%d @ %g] Received an APPLICATION Msg sent from robot %d at %g to robot %d (size %d bytes)\n", 
           id, current_time_slice, msg_header.from, msg_header.timestamp, msg_header.to, msg_header.size);
   fprintf(stderr, "     Msg content: [%s]\n", msg_content + sizeof(ApplicationPacket));
}

void NetworkApplication::SendReceivedDataToRobotApp(Real current_time_slice, int from, string content)
{
   GenerateMsg(&received_data, msg_size, from, id, content, current_time_slice);
   ServerSendDataToSocket(socket, received_data, msg_size);
};

