#include "robot-simulator.h" 

void BuildSimulationInformationPacket(char **msg, SInt32 &msg_size, const SInt32 robot_num,
					const Real simulation_step, const Real simulation_time,
					const char *sim_name, const Real stop_time,
					const Real f_arena_x, const Real f_arena_y, const Real f_arena_z)
  {
    SimulationInformationPacket info;

    info.type            = SIMULATION_INFORMATION;
    info.robot_num       = robot_num;
    info.simulation_step = simulation_step;
    info.start_time      = simulation_time;
    info.stop_time       = stop_time;
    info.arena_x         = f_arena_x;
    info.arena_y         = f_arena_y;
    info.arena_z         = f_arena_z;
    strcpy(info.simulator_name, sim_name);

    memcpy(*msg, (void *)&info, sizeof(SimulationInformationPacket));

    msg_size = sizeof(SimulationInformationPacket);
  }

  /************************************************/
  /************************************************/

  void BuildSimulationInformationPacket(char *msg, SInt32 &msg_size, const SInt32 robot_num,
					const Real simulation_step, const Real simulation_time,
					const char *sim_name, const Real stop_time,
					const Real f_arena_x, const Real f_arena_y, const Real f_arena_z)
  {
    SimulationInformationPacket info;

    info.type            = SIMULATION_INFORMATION;
    info.robot_num       = robot_num;
    info.simulation_step = simulation_step;
    info.start_time      = simulation_time;
    info.stop_time       = stop_time;
    info.arena_x         = f_arena_x;
    info.arena_y         = f_arena_y;
    info.arena_z         = f_arena_z;
    strcpy(info.simulator_name, sim_name);

    memcpy(msg, (void *)&info, sizeof(SimulationInformationPacket));

    msg_size = sizeof(SimulationInformationPacket);
  }

int main(int argc, char *argv[])
{
   if (argc != 4)
     {
       fprintf(stderr, "Usage: %s <server> <port_base> <robot_num>\n", argv[0]);
       exit(-1);
     }

   char *server_host = argv[1];
   int port_base = atoi(argv[2]);
   int robot_num = atoi(argv[3]);
   static const Real simulation_step = 0.1;
   Real current_time_slice = 0.0;
   Real simulationStop = 1000.0;
   Real arenaX = 1000.0;
   Real arenaY = 1000.0;
   Real arenaZ = 1.0;

   srand(12345);

   char num_msgs;
   int msg_size;
   char *msg = new char[max_data];
     
   Position *positions = new Position[robot_num];

   Socket socket_scheduler;
   Socket *socket_application = new Socket[robot_num];

   OpenBlockingUDPSocketClientSide(socket_scheduler, port_base, server_host);

   for(int i=0; i < robot_num; i++)
     OpenBlockingUDPSocketClientSide(socket_application[i], port_base + (i + 1), server_host);


   // send simulation information packet
   BuildSimulationInformationPacket(&msg, msg_size, robot_num, simulation_step, current_time_slice, "ARGoS",simulationStop,arenaX,arenaY,arenaZ);
   ClientSendDataToSocket(socket_scheduler, msg, msg_size);

   // get an ack from the network simulator (this serves to sync the client/server processes)
   ClientReceiveDataFromSocketBlocking(socket_scheduler, &msg, msg_size, max_data);
   if( msg_size > 0 )
     {
       NetworkSimInfoPacket *info = (NetworkSimInfoPacket *)msg;
       if(info->type != NETWORK_SIM_INFORMATION)
	 {
	   fprintf(stderr, "[NETWORK_SIM_INFORMATION] WRONG PACKET TYPE: %d\n", info->type);
	   exit(-1);
	 }
       else
	 fprintf(stderr, "Received Start info packet from network simulator %s\n", info->network_sim);
     }
   
   // send start packet from robot process to network nodes
   for(int i=0; i < robot_num; i++)
     {
       BuildRobotStartPacket(&msg, msg_size, i, "Foot-bot");
       ClientSendDataToSocket(socket_application[i], msg, msg_size);
     }

   while (true) 
     {
       GenerateRandomPositions(positions, robot_num, 10., 10., 5.);
       
       // wait for the synch ack from the network scheduler
       ClientReceiveDataFromSocketBlocking(socket_scheduler, &msg, msg_size, max_data);
       fprintf(stderr, "[%g] Received ack of %d bytes [%d]\n", current_time_slice, msg_size, (int)msg[0]);

       // send positions for this time step
       BuildRobotPositionsPacket(&msg, msg_size, robot_num, positions, current_time_slice);
       ClientSendDataToSocket(socket_scheduler, msg, msg_size);

       // robot wifi sensors receive messages
       for(int i=0; i < robot_num; i++)
       {
          while (ClientReceiveDataFromSocketNonBlocking(socket_application[i], &msg, msg_size, max_data) > 0)
          {
             ApplicationPacket *appl = (ApplicationPacket *)msg;
             char *message = msg + sizeof(ApplicationPacket);
             if(appl->type != APPLICATION_DATA)
             {
                fprintf(stderr, "WRONG PACKET TYPE: %d\n", appl->type);
                exit(-1);
             }
             else
             {
                fprintf(stderr, "[%d @ %g]Received %d bytes from robot %d: %s\n", i, current_time_slice, appl->size, appl->from, message);
             }
          }
        }

        // robot wifi actuators send messages
        for(int i=0; i < robot_num; i++)
        {
 	   GenerateRandomMsg(&msg, msg_size, i, robot_num, current_time_slice);
           num_msgs = ( msg_size > 0) ? 1 : 0;
           ClientSendDataToSocket(socket_application[i], &num_msgs, 1);
	   if( msg_size > 0)
	      ClientSendDataToSocket(socket_application[i], msg, msg_size);
        }

        // simulate calculations of physics engine
        usleep(1000000);

        // update time
        current_time_slice += simulation_step;
     }

   signal(SIGCHLD,SIG_IGN);

   return(0);
}
