/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2010 UCM
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author:  Delfín Rupérez Cañas <delfinrc@fdi.ucm.es>
 *          
 */

#include "antor-dlr-queue.h"
#include <algorithm>
#include <functional>
#include "ns3/ipv4-route.h"
#include "ns3/socket.h"
#include "ns3/log.h"

NS_LOG_COMPONENT_DEFINE ("AntOR-DLR-Queue");

namespace ns3
{
namespace antordlr
{
uint32_t
RequestQueue::GetSize ()
{
  Purge ();
  return m_queue.size ();
}

bool
RequestQueue::Enqueue (QueueEntry & entry)
{
  Purge ();
  for (std::vector<QueueEntry>::const_iterator i = m_queue.begin (); i
      != m_queue.end (); ++i)
    {
      if ((i->GetPacket ()->GetUid () == entry.GetPacket ()->GetUid ())
          && (i->GetIpv4Header ().GetDestination ()
              == entry.GetIpv4Header ().GetDestination ()))
        return false;
    }
  entry.SetExpireTime (m_queueTimeout);
  if (m_queue.size () == m_maxLen)
    {
      Drop (m_queue.front (), "Drop the most aged packet"); // Drop the most aged packet
      m_queue.erase (m_queue.begin ());
    }
  m_queue.push_back (entry);
  return true;
}

void
RequestQueue::DropPacketWithDst (Ipv4Address dst)
{
  NS_LOG_FUNCTION (this << dst);
  Purge ();
  const Ipv4Address addr = dst;
  for (std::vector<QueueEntry>::iterator i = m_queue.begin (); i
      != m_queue.end (); ++i)
    {
      if (IsEqual (*i, dst))
        {
          Drop (*i, "DropPacketWithDst ");
        }
    }
  m_queue.erase (std::remove_if (m_queue.begin (), m_queue.end (),
      std::bind2nd (std::ptr_fun (RequestQueue::IsEqual), dst)), m_queue.end ());
}
bool RequestQueue::GetPackets (Ipv4Address dst, std::vector<QueueEntry>  temp){
	   bool cond = false;
	    for (std::vector<QueueEntry>::iterator i = m_queue.begin (); i != m_queue.end (); ++i)
	      {
	  	  if (i->GetIpv4Header ().GetDestination () == dst){
	  		  temp.push_back(*i);
	  		  cond = true;
	  	  }
	      }
return cond;

}
bool RequestQueue::UpdatePacketHops(Ipv4Address dst,uint32_t hops) {
	 bool cond = false;
	 if (m_queue.empty())
		 return cond;
	 for (std::vector<QueueEntry>::iterator i = m_queue.begin (); i != m_queue.end (); ++i)
		      {
		  	  if (i->GetIpv4Header ().GetDestination () == dst){
		  		  i->SetHops(hops);
		  		  cond = true;
		  	  }
		      }
	return cond;


}
bool
RequestQueue::Dequeue (Ipv4Address dst, QueueEntry & entry)
{
  Purge ();
/*
  //FIXME
  std::vector<QueueEntry> temp;
    bool cond = false;
    for (std::vector<QueueEntry>::iterator i = m_queue.begin (); i != m_queue.end (); ++i)
      {
  	  if (i->GetIpv4Header ().GetDestination () == dst){
  		  temp.push_back(*i);
  		  cond = true;
  	  }
      }

  	if (cond){
       uint32_t idx = 0;
       uint32_t max_hops = 0;
       for (uint32_t k=0; k < temp.size(); k++){
    	 // std::cout <<  (uint32_t) temp[k].GetHops() << std::endl;
    	//if ((uint32_t) temp[k].GetHops()> 1){
    	//	std::cout <<  (uint32_t) temp[k].GetHops() << std::endl;
    	//}
    	if ((uint32_t) temp[k].GetHops() > max_hops){
      	entry = temp[k];
      	idx = k;
      	max_hops =(uint32_t)  temp[k].GetHops();

        }
       }

       m_queue.erase(m_queue.begin() + idx);
  	  return true;
  	}
  	return false;
}
*/

  for (std::vector<QueueEntry>::iterator i = m_queue.begin (); i != m_queue.end (); ++i)
    {
      if (i->GetIpv4Header ().GetDestination () == dst)
        {
          entry = *i;
          m_queue.erase (i);
          return true;
        }
    }
  return false;

 }

bool
RequestQueue::Find (Ipv4Address dst)
{
  for (std::vector<QueueEntry>::const_iterator i = m_queue.begin (); i
      != m_queue.end (); ++i)
    {
      if (i->GetIpv4Header ().GetDestination () == dst)
        return true;
    }
  return false;
}

struct IsExpired
{
  bool
  operator() (QueueEntry const & e) const
  {
    return (e.GetExpireTime () < Seconds (0));
  }
};

void
RequestQueue::Purge ()
{
  IsExpired pred;
  for (std::vector<QueueEntry>::iterator i = m_queue.begin (); i
      != m_queue.end (); ++i)
    {
      if (pred (*i))
        {
          Drop (*i, "Drop outdated packet ");
        }
    }
  m_queue.erase (std::remove_if (m_queue.begin (), m_queue.end (), pred),
      m_queue.end ());
}

void
RequestQueue::Drop (QueueEntry en, std::string reason)
{
  NS_LOG_LOGIC (reason << en.GetPacket ()->GetUid () << " " << en.GetIpv4Header ().GetDestination ());
  en.GetErrorCallback () (en.GetPacket (), en.GetIpv4Header (),
      Socket::ERROR_NOROUTETOHOST);
  return;
}

}
}
